/*************************************************************************
*
*   This module contains the wire byte swapping routines for PING
*
*   revision: $Id: //icaclient/release/ica800unix/src/examples/vc/client/vdping/unix/pingwire.c#1 $
*
*   Copyright 2001, 2009 Citrix Systems, Inc.  All Rights Reserved.
*
*   Citrix hereby permits you to use, reproduce, modify, display, compile,
*   sublicense, and distribute all or any portion of the sample code contained
*   in this file, provided that you remove this Copyright attribution from any
*   modified version of this file.
*
*   THIS SAMPLE CODE IS PROVIDED BY CITRIX "AS IS" AND ANY EXPRESS OR IMPLIED
*   WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
*   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
*
*************************************************************************/

#include <citrix.h>
#include <cwire.h>
#include <cmacro.h>

#include "vdping.h"
#include "pingwire.h"

/****************************************************************************
 *
 *  Marshall_Write_VDPING_C2H
 *
 *    marshalls the client to host structure onto the wire to be sent to the
 *    server. This routine is not called on littlendian platforms
 *
 * ENTRY:
 *    pointer to wire buffer containing the VDPING_C2H structure in client format,
 *    not necessarily packed on byte boundries.
 *
 * EXIT:
 *    returns the packed size of the data written to the wire
 *
 ***************************************************************************/

USHORT Marshall_Write_VDPING_C2H (PVDPING_C2H input, USHORT inputSize)
{
	LPBYTE writePtr = (LPBYTE)input;

	writePtr += Marshall_Write_VD_C2H ((PVD_C2H) &input->Header, packed_sizeof_VDPING_C2H);

	ctx_wire_write16(&input->usMaxDataSize, writePtr);
	ctx_wire_write16(&input->usPingCount, writePtr);

	return ((USHORT)(writePtr - (LPBYTE)input));
}

/****************************************************************************
 *
 *  Marshall_Write_PING
 *
 *    marshalls the client to host structure onto the wire to be sent to the
 *    server. This routine is not called on littlendian platforms
 *
 * ENTRY:
 *    pointer to wire buffer containing the VDPING_C2H structure in client format,
 *    not necessarily packed on byte boundries.
 *
 * EXIT:
 *    returns the packed size of the data written to the wire
 *
 ***************************************************************************/

USHORT Marshall_Write_PING (PPING input, USHORT inputSize)
{
	LPBYTE writePtr = (LPBYTE)input;

	ctx_wire_write16(&input->uSign, writePtr);
	ctx_wire_write16(&input->uType, writePtr);
	ctx_wire_write16(&input->uLen, writePtr);
	ctx_wire_write16(&input->uCounter, writePtr);
	ctx_wire_write32(&input->ulServerMS, writePtr);
	ctx_wire_write32(&input->ulClientMS, writePtr);

	return ((USHORT)(writePtr - (LPBYTE)input));
}

/****************************************************************************
 *
 *  Marshall_Read_PING
 *
 * ENTRY:
 *    void pointer to wire buffer containing the data (input), and the
 *    buffer that the data is to be swapped into (output).
 *
 ***************************************************************************/

void Marshall_Read_PING(LPVOID input, LPVOID output)
{
        LPBYTE readPtr = (LPBYTE)input;
        PPING writePtr = (PPING)output;

        ctx_wire_read16(readPtr, &writePtr->uSign);
        ctx_wire_read16(readPtr, &writePtr->uType);
        ctx_wire_read16(readPtr, &writePtr->uLen);
        ctx_wire_read16(readPtr, &writePtr->uCounter);
        ctx_wire_read32(readPtr, &writePtr->ulServerMS);
        ctx_wire_read32(readPtr, &writePtr->ulClientMS);
}


